import sys

#
# error reporting
#

class ParserError(Exception) :
    pass

def error(msg) :
    raise ParserError(msg)


#
# the input string, and the current position
#

input = ''
position = 0

def setInput(s) :
    global input, position
    input = s
    position = 0

def next() :
    global position
    if position >= len(input) :
        error('unexpected end of input')
    c = input[position]
    position = position + 1
    return c

def save() :
    return position

def restore(p) :
    global position
    position = p


#
# parseDigit, parseNumeral
#

def parseDigit() :
    c = next()
    code = ord(c)
    if (code < 48) or (code > 57) :
        error('invalid digit: ' + c)
    return code - 48

def parseNumeral() :
    result = parseDigit()
    while True :
        p = save()
        try :
            d = parseDigit()
            result = result*10 + d
        except ParserError :
            restore(p)
            break
    return result


#
# parseParenthesized
#

def parseParenthesized() :
    parseOpeningParenthesis()
    x = parseExpr()
    parseClosingParenthesis()
    return x

def parseOpeningParenthesis() :
    c = next()
    if c != '(' :
        error('expected (, but received: ' + c)

def parseClosingParenthesis() :
    c = next()
    if c != ')' :
        error('expected ), but received: ' + c)


#
# parseFactor
#

def parseFactor() :
    p = save()
    try :
        return parseNumeral()
    except ParserError :
        restore(p)
        return parseParenthesized()


#
# parseTerm
#

def parseTerm() :
    a = parseFactor()
    while True :
        p = save()
        try :
            parseStar()
            b = parseFactor()
            a = a * b
        except ParserError :
            restore(p)
            break
    return a

def parseStar() :
    c = next()
    if c != '*' :
        error('expected *, but received: ' + c)


#
# parseExpr
#

def parseExpr() :
    a = parseTerm()
    while True :
        p = save()
        try :
            op = parsePlusOrMinus()
            b = parseTerm()
            a = applyPlusOrMinus(op, a, b)
        except ParserError :
            restore(p)
            break
    return a

def parsePlusOrMinus() :
    c = next()
    if (c != '+') and (c != '-') :
        error('expected one of + -, but received: ' + c)
    return c

def applyPlusOrMinus(op, a, b) :
    if op == '+' : return a + b
    elif op == '-' : return a - b
    else :
        error('invalid operator: ' + op)


#
# main loop
#

def processLine(line) :
    setInput(line)
    x = parseExpr()
    print x

    if position != len(input) :
        unparsed = input[position:]
        error('unparsed leftover: ' + unparsed)

def readLine() :
    try :
        return raw_input('> ')
    except EOFError :
        return None

def main() :
    while True :
        line = readLine()
        if (line == None) or (line == 'quit') :
            print 'Bye!'
            break
        try :
            processLine(line)
        except ParserError, e :
            print e

main()
