
#
# error reporting
#

class ParserError(Exception) :
    pass

def error(msg) :
    raise ParserError(msg)


#
# primitives
#

_input = None
_position = 0

def setInput(s) :
    global _input, _position
    _input = s
    _position = 0

def next() :
    global _position
    if _position >= len(_input) :
        error('unexpected end of input')
    x = _input[_position]
    _position = _position + 1
    return x

def save() :
    return _position

def restore(p) :
    global _position
    _position = p

def remaining() :
    return _input[_position:]


#
# combinators
#

def condition(f, modifier=lambda x : x) :
    def proc() :
        x = next()
        if not f(x) :
            error('invalid input: %s' + str(x))
        return modifier(x)
    return proc

def char(something, modifier=lambda x : x) :
    def f(x) :
        return x == something
    return condition(f, modifier)

def zeroOrMore(parser, modifier=lambda x : x) :
    def proc() :
        result = []
        while True :
            p = save()
            try :
                x = parser()
                result.append(x)
            except ParserError :
                restore(p)
                break
        return modifier(result)
    return proc

def oneOrMore(parser, modifier=lambda x : x) :
    def proc() :
        result = [parser()] + zeroOrMore(parser)()
        return modifier(result)
    return proc

def sequence(parsers, modifier=lambda x : x) :
    def proc() :
        result = []
        for parser in parsers :
            result.append(parser())
        return modifier(result)
    return proc

def choice(parsers) :
    def proc() :
        p = save()
        for parser in parsers :
            try :
                return parser()
            except ParserError :
                restore(p)
        error('parse error')
    return proc
